#include "sub_s.hh"
#include "pub_c.hh"


class Client: public _sk_Subscriber
{
    private:
	CORBA::String_var SubName;
    public:
	Client(const char *name=NULL) : _sk_Subscriber(name) {
		SubName = CORBA::string_dup(name);
	} 

	~Client() {}

	char* get_name(){ return CORBA::string_dup(SubName); }

	void receive_update(const Quote& q) {
		cout << "Received update: " << endl;
		cout << q << endl;
	}
};


int main(int argc, char *const *argv)
{
	CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
	CORBA::BOA_var boa = orb->BOA_init(argc, argv);

	CORBA::BOA::scope(CORBA::BOA::SCOPE_LOCAL); // Create Transient client

	Client client(argv[1]);
	boa->obj_is_ready(&client);

	Publisher_var publisher;

	try {
		publisher = Publisher::_bind();
	}
	catch (const CORBA::Exception& e) {
		cout << "Client: Unable to bind to publisher" << endl;
		cout << e << endl;
		return 1;
	}

	cout << "Successfully bound to publisher" << endl;

	try {
		publisher->add_subscriber(&client);

		SubscriberList_ptr s = publisher->get_subscribers();
		for (CORBA::ULong i = 0; i < s->length(); i++)
		{
			cout << (*s)[i]->get_name() << endl;
		}
		SubscriberList::_release(s);
	}
	catch (const CORBA::Exception& e) {
		cout << "Client: Unable to add subscriber" << endl;
		cout << e << endl;
		return 1;
	}

	boa->impl_is_ready();	// Wait for incoming calls
	return 0;
}
